unit Queue_Unit;

interface

uses
  SysUtils;

type
  QData = string;
  PQu = ^Qu;
  Qu = record
    Next: PQu;
    Inf: QData;
  end;

  TQueue = class
    private
      Head: PQu;
      Tail: PQu;
      Look: PQu;
      Length: Integer;
    public
      constructor Create;
      destructor Destroy;
      function Pop: QData;
      procedure Push(C: QData);
      function Empty: Boolean;
      function GetLength: Integer;
      function LookHead: QData;
      function LookTail: QData;
      procedure ResetLook;
      function LookNext: QData;
      function LookEnd: Boolean;
      function Delete(Number: Integer): Boolean;
      procedure Clear;
  end;

implementation

procedure TQueue.Clear;
  var t: QData;
begin
while not self.Empty do
  t:=self.Pop;
Head:=nil;
Tail:=nil;
Look:=nil;
Length:=0;
end;

function TQueue.LookTail: QData;
begin
  Result:= Tail^.Inf;
end;

function TQueue.Delete(Number: Integer): Boolean;
  var temp, t2: PQu;
    i: Integer;
begin
  if ( (Number<=Length) and (Number>0) )
    then
      begin
        if Number=1
          then
            begin
              t2:=Head;
              Head:=Head^.Next;
            end
          else
            begin
              temp:=Head;
              i:=1;
              while ( i<(Number-1) ) do
                begin
                  i:=i+1;
                  temp:=temp^.Next;
                end;
              t2:=temp^.Next;
              temp^.Next:=t2^.Next;
            end;
        Length:=Length-1;
        Dispose(t2);
        Result:=true;
      end
    else Result:=false;
end;

function TQueue.LookEnd: Boolean;
begin
  Result:= Look=nil;
end;

function TQueue.LookNext: QData;
  var C: QData;
begin
  if Look<>nil
    then
      begin
        C:=self.Look^.Inf;
        Look:=Look^.Next;
      end;
  Result:=C;
end;

procedure TQueue.ResetLook;
begin
  Look:=Head;
end;

function TQueue.LookHead: QData;
begin
  Result:=Head^.Inf;
end;

destructor TQueue.Destroy;
  var t: QData;
begin
  while not self.Empty do
    t:=self.Pop;
end;

function TQueue.GetLength: Integer;
begin
  Result:=Length;
end;

function TQueue.Pop: QData;
  var N: PQu; C: QData;
begin
if Head<>nil
  then
    begin
      N:=Head;
      C:=N^.Inf;
      Head:=Head^.Next;
      if Head=nil
        then Tail:=nil;
      Dispose(N);
      Dec(Length);
    end;
Result:=C;
end;

procedure TQueue.Push(C: QData);
  var N: PQu;
begin
  New(N);
  N^.Inf:=C;
  N^.Next:=nil;
  if Head=nil
    then
      begin
        Head:=N;
        Tail:=N;
      end
    else
      begin
        Tail^.Next:=N;
        Tail:=N;
      end;
  Inc(Length);
end;

function TQueue.Empty: Boolean;
begin
  if Length=0
    then Result:=true
    else Result:=false;
end;

constructor TQueue.Create;
begin
  Head:=nil;
  Tail:=nil;
  Look:=nil;
  Length:=0;
end;

end.
 